import { OrgData } from '@ibiz-template/core';
import { ModelUtil } from '@ibiz-template/model';
import { UnauthorizedHandler } from '../unauthorized-handler/unauthorized-handler';

/**
 * 预加载应用级引用插件
 *
 * @author chitanda
 * @date 2022-10-31 16:10:57
 * @return {*}  {Promise<void>}
 */
async function loadAppPlugins(): Promise<void> {
  const modelService = await ModelUtil.getModelService();
  const { app } = modelService;
  const pluginRefs = app.getAllPSAppPFPluginRefs();
  const all = pluginRefs?.map(async pluginRef => {
    if (
      pluginRef &&
      pluginRef.runtimeObject &&
      pluginRef.rTObjectName &&
      pluginRef.rTObjectRepo
    ) {
      const config = JSON.parse(pluginRef.rTObjectRepo);
      if (config && config.app === true) {
        await ibiz.plugin.loadPluginRef(pluginRef);
      }
    }
  });
  if (all) {
    await Promise.all(all);
  }
}

/**
 * 初始化模型
 *
 * @author chitanda
 * @date 2022-07-20 20:07:58
 * @return {*}  {Promise<void>}
 */
async function loadModel(): Promise<void> {
  await ModelUtil.create(async modelPath => {
    const url = `${ibiz.env.remoteModelUrl}${modelPath}`;
    const res = await ibiz.net.get(url);
    if (res.ok) {
      return res.data as IModel;
    }
    return {};
  });
  await loadAppPlugins();
}

/**
 * 加载应用数据
 *
 * @author chitanda
 * @date 2022-07-20 20:07:50
 * @return {*}  {Promise<void>}
 */
async function loadAppData(): Promise<void> {
  const res = await ibiz.net.get('/appdata');
  if (res.ok) {
    ibiz.appData = res.data;
  }
}

/**
 * 加载组织数据
 *
 * @author chitanda
 * @date 2022-07-20 20:07:44
 * @return {*}  {Promise<void>}
 */
async function loadOrgData(): Promise<void> {
  const res = await ibiz.net.get(`/uaa/getbydcsystem/${ibiz.env.dcSystem}`);
  if (res.ok) {
    const orgDataItems = res.data as OrgData[];
    if (orgDataItems) {
      const [data] = orgDataItems;
      ibiz.orgData = data;
    }
  }
}

/**
 * 应用参数初始化
 *
 * @author chitanda
 * @date 2022-07-20 19:07:54
 * @return {*}  {Promise<void>}
 */
async function appInit(): Promise<void> {
  try {
    await loadOrgData();
    await loadAppData();
    await loadModel();
    // 设置权限服务需要的appData里的数据
    await ibiz.authority.init();
  } catch (error) {
    const { response } = error as IData;
    if (response?.status === 401) {
      await UnauthorizedHandler.handle();
    } else {
      ibiz.log.error(error);
    }
  }
}

/**
 * 应用权限守卫
 *
 * @author chitanda
 * @date 2022-10-28 10:10:29
 * @export
 * @return {*}  {Promise<boolean>}
 */
export async function AuthGuard(): Promise<boolean> {
  await appInit();
  return true;
}
